package org.jboss.cache.loader;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.jboss.cache.config.CacheLoaderConfig.IndividualCacheLoaderConfig;

import java.util.Properties;

/**
 * Builds the different SQLs needed by <tt>JDBCCacheLoader</tt>.
 *
 * @author Mircea.Markus@iquestint.com
 * @version 1.0
 */
public class JDBCCacheLoaderConfig extends AdjListJDBCCacheLoaderConfig
{

    private static final long serialVersionUID = -8371846151643130271L;

    private static final Log log = LogFactory.getLog(JDBCCacheLoaderConfig.class);

    private String deleteNode;
    private String recursiveChildren;
    private String nodeCountSql;


    public JDBCCacheLoaderConfig(IndividualCacheLoaderConfig base)
    {
      super(base);
      setClassName(JDBCCacheLoader.class.getName());
    }


    public void setProperties(Properties props)
    {
        super.setProperties(props);
        String sqlConcat = props.getProperty("cache.jdbc.sql-concat");
        if (sqlConcat == null) {
            log.info("Missiing JDBCCacheLoader config 'cache.jdbc.sql-concat', using default value:'concat(1,2)'");
            sqlConcat = "concat(1,2)";
        }
        String startingWith = sqlConcat.replace('1', '?').replace("2", "'%'"); //concat(?, '%')
        String appendSepparator = sqlConcat.replace("1", fqnColumn).replace("2", "'/'"); //concat(fqnColumn, '/')
        deleteNode = "delete from " + table + " where " + appendSepparator + " like " + startingWith;
        recursiveChildren = "select " + fqnColumn + "," + nodeColumn + " from " + table + " where " + appendSepparator + " like " + startingWith;
        nodeCountSql = "select count(*) from " + table;
    }


    /**
     * Returns the sql string for removing a node and all its children.
     */
    public String getDeleteNodeSql()
    {
        return deleteNode;
    }

    /**
     * Returns an sql that will return a node and all its children.
     */
    public String getRecursiveChildrenSql()
    {
        return recursiveChildren;
    }


    /**
     * Returns an sql that will count all the persisted node.
     */
    public String getNodeCountSql()
    {
        return nodeCountSql;
    }
}
