/*
 * JBoss, Home of Professional Open Source
 *
 * Distributable under LGPL license.
 * See terms of license at gnu.org.
 */
package org.jboss.cache.marshall;

import java.io.IOException;
import java.io.InputStream;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.OutputStream;

/**
 * Factory class for creating object output and inut streams, to allow for multiple mechanisms of serialization.
 * Java serialization is the only supported mechanism at this point.
 *
 * @author <a href="mailto:manik@jboss.org">Manik Surtani (manik@jboss.org)</a>
 * @author <a href="mailto:clebert.suconic@jboss.org">Clebert Suconic</a>
 * @author <a href="mailto:galder.zamarreno@jboss.com">Galder Zamarreno</a>
 */
public class ObjectSerializationFactory
{
   static ObjectStreamFactory factory = new JavaObjectStreamFactory();

   /*
   static
   {
      // start with the NEW property
      String propString = System.getProperty("jboss.serialization");
      if (propString == null)
      {
         // and now check legacy
         propString = System.getProperty("serialization.jboss");
         if (propString != null)
            log.info("The system property 'serialization.jboss' is deprecated and may be removed from future releases.  Please use 'jboss.serialization' instead.");
      }      
      useJBossSerialization = false; // default.
      if (propString != null) useJBossSerialization = Boolean.valueOf(propString);

      try
      {
         if (useJBossSerialization)
         {
            factory = (ObjectStreamFactory) Class.forName("org.jboss.cache.marshall.JBossObjectStreamFactory").newInstance();
         }
      }
      catch (Exception e)
      {
         log.error("Unable to load JBossObjectStreamFactory.  Perhaps jboss-serialization jar not loaded?", e);
         log.error("Falling back to java serialization.");
      }
   }
   */

   public static ObjectOutputStream createObjectOutputStream(OutputStream out) throws IOException
   {
      return factory.createObjectOutputStream(out);
   }

   public static ObjectInputStream createObjectInputStream(byte[] bytes) throws IOException
   {
      return factory.createObjectInputStream(bytes);
   }

   public static ObjectInputStream createObjectInputStream(InputStream in) throws IOException
   {
      return factory.createObjectInputStream(in);
   }

//   public static boolean useJBossSerialization()
//   {
//      return useJBossSerialization;
//   }
}
