package org.jboss.cache.eviction;


import org.jboss.cache.CacheSPI;
import org.jboss.cache.DefaultCacheFactory;
import org.jboss.cache.Fqn;
import org.jboss.cache.lock.IsolationLevel;
import org.jboss.cache.misc.TestingUtil;
import static org.testng.AssertJUnit.*;
import org.testng.annotations.AfterMethod;
import org.testng.annotations.BeforeMethod;
import org.testng.annotations.Test;

/**
 * Unit tests for LRU Policy.
 *
 * @author Ben Wang, Feb 11, 2004
 * @author Daniel Huang - dhuang@jboss.org
 * @version $Revision: 4880 $
 */
@Test(groups = {"functional"})
public class NullEvictionPolicyTest
{
   CacheSPI<Object, Object> cache_;

   @BeforeMethod(alwaysRun = true)
   public void setUp() throws Exception
   {
      cache_ = null;
   }

   @AfterMethod(alwaysRun = true)
   public void tearDown() throws Exception
   {
      if (cache_ != null)
      {
         cache_.stop();
         cache_.destroy();
      }      
   }

   /**
    * Builds a cache was null eviction by default and in "/test" region,
    * LRU in "/lru" region.  Does a mix of puts/reads/removes, wakes for
    * eviction thread to kick in, checks that nothing was evicted from the
    * null policy regions but was from lru region.
    */
   public void testEviction()
   {      
      cache_ = (CacheSPI<Object, Object>) new DefaultCacheFactory().createCache("META-INF/local-null-eviction-service.xml", false);// read in generic local xml
      cache_.getConfiguration().setTransactionManagerLookupClass("org.jboss.cache.transaction.DummyTransactionManagerLookup");
      cache_.getConfiguration().setIsolationLevel(IsolationLevel.SERIALIZABLE);
      cache_.start();
      
      String dfltRootStr = "/a/";
      String testRootStr = "/test/";
      String lruRootStr = "/lru/";
      
      for (int i = 0; i < 20; i++)
      {                 
         if (i % 1 == 0 || i % 3 == 0)
         {
            Fqn dflt = Fqn.fromString(dfltRootStr + i);
            Fqn test = Fqn.fromString(testRootStr + i);
            Fqn lru = Fqn.fromString(lruRootStr + i);
            cache_.put(dflt, "key", "value");
            cache_.put(test, "key", "value");
            cache_.put(lru, "key", "value");
         }
         else
         {
            Fqn dflt = Fqn.fromString(dfltRootStr + (i - 1));
            Fqn test = Fqn.fromString(testRootStr + (i - 1));
            Fqn lru = Fqn.fromString(lruRootStr + (i - 1));
            if (i % 2 == 0)
            {
               assertEquals("value", cache_.get(dflt, "key"));
               assertEquals("value", cache_.get(test, "key"));
               assertEquals("value", cache_.get(lru, "key"));
            }
            else
            {
               cache_.removeNode(dflt);
               cache_.removeNode(test);
               cache_.removeNode(lru);
            }
         }
      }
      
      TestingUtil.sleepThread(2500);
      
      for (int i = 0; i < 20; i++)
      {
         Fqn dflt = Fqn.fromString(dfltRootStr + i);
         Fqn test = Fqn.fromString(testRootStr + i);
         Fqn lru = Fqn.fromString(lruRootStr + i);
         
         if (i % 1 == 0)
         {
            assertEquals("value", cache_.get(dflt, "key"));
            assertEquals("value", cache_.get(test, "key"));
            assertNull(cache_.get(lru, "key"));
         }
         else if (i % 3 == 0)
         {
            assertNull(cache_.get(dflt, "key"));
            assertNull(cache_.get(test, "key"));
            assertNull(cache_.get(lru, "key"));
         }
      }
   }

}
